/**
 * The MySensors Arduino library handles the wireless radio link and protocol
 * between your home built sensors/actuators and HA controller of choice.
 * The sensors forms a self healing radio network with optional repeaters. Each
 * repeater and gateway builds a routing tables in EEPROM which keeps track of the
 * network topology allowing messages to be routed to nodes.
 *
 * Created by Henrik Ekblad <henrik.ekblad@mysensors.org>
 * Copyright (C) 2013-2015 Sensnology AB
 * Full contributor list: https://github.com/mysensors/Arduino/graphs/contributors
 *
 * Documentation: http://www.mysensors.org
 * Support Forum: http://forum.mysensors.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 *******************************
 *
 * 
 */
 
  /*
 arduino nano 328
*/

#include <SPI.h>
#include <MySensor.h>  

#define CHILD_ID_HVACHC 5
#define NODE_REPEAT false
#define NODE_ID 25

MySensor gw;

MyMessage msgTemp(CHILD_ID_HVACHC, V_TEMP);
MyMessage msgSetpointHeat(CHILD_ID_HVACHC, V_HVAC_SETPOINT_HEAT);
MyMessage msgSetpointCool(CHILD_ID_HVACHC, V_HVAC_SETPOINT_COOL);
MyMessage msgSetpointFlowState(CHILD_ID_HVACHC, V_HVAC_FLOW_STATE);
MyMessage msgSetpointFlowMode(CHILD_ID_HVACHC, V_HVAC_FLOW_MODE);
MyMessage msgSetpointFanStatus(CHILD_ID_HVACHC, V_HVAC_FAN_STATUS);
MyMessage msgSetpointModeState(CHILD_ID_HVACHC, V_HVAC_MODE_STATE);
MyMessage msgSetpointEnergyMode(CHILD_ID_HVACHC, V_HVAC_ENERGY_MODE);


void setup()  
{ 
  gw.begin(incomingMessage, NODE_ID, NODE_REPEAT);

  gw.sendSketchInfo("HVAC Thermostat", "1.0");

  gw.present(CHILD_ID_HVACHC, S_HVACHC);

  gw.send(msgTemp.set(23.8, 1));
  gw.send(msgSetpointHeat.set(23.5, 1));
  gw.send(msgSetpointCool.set(20.5, 1));
  gw.send(msgSetpointFlowState.set("HeatOn")); // Off, AutoChangeOver, HeatOn, CoolOn, 
  gw.send(msgSetpointFlowMode.set("ContinuousOn")); // Auto, ContinuousOn, PeriodicOn
  gw.send(msgSetpointFanStatus.set("On")); // On, Off
  gw.send(msgSetpointModeState.set("Idle")); // Off, Heating, Cooling, FanOnly, Idle, Error! 
  gw.send(msgSetpointEnergyMode.set("Normal")); //EnergySavingsMode, Normal
  
}

void loop()      
{  
  gw.process();
}

void incomingMessage(const MyMessage &message)
{
  if(message.sensor == CHILD_ID_HVACHC)
  {  
    if (message.type == V_HVAC_SETPOINT_HEAT) {
      Serial.print( "V_HVAC_SETPOINT_HEAT setpoint received: " );      
      Serial.println( message.data );      
    }
    if (message.type == V_HVAC_SETPOINT_COOL) {
      Serial.print( "V_HVAC_SETPOINT_COOL setpoint received: " );      
      Serial.println( message.data );      
    }
    if (message.type == V_HVAC_FLOW_STATE ) {
      Serial.print( "V_HVAC_FLOW_STATE received: " );      
      Serial.println( message.data );
      if(String(message.data)== "Off"){  
        gw.send(msgSetpointModeState.set("Off"));
        gw.send(msgSetpointFanStatus.set("Off"));
      }
      else if(String(message.data)== "HeatOn"){  
        gw.send(msgSetpointModeState.set("Heating"));
        gw.send(msgSetpointFanStatus.set("On"));
      }
      else if(String(message.data)== "CoolOn"){  
        gw.send(msgSetpointModeState.set("Cooling")); 
        gw.send(msgSetpointFanStatus.set("On"));
      }  
      else{
        gw.send(msgSetpointModeState.set("Idle"));   
        gw.send(msgSetpointFanStatus.set("Off"));
      }   
    }
    if (message.type == V_HVAC_FLOW_MODE) {
      Serial.print( "V_HVAC_FLOW_MODE received: " );      
      Serial.println( message.data );      
    }
    if (message.type == V_HVAC_ENERGY_MODE) {
      Serial.print( "V_HVAC_ENERGY_MODE received: " );      
      Serial.println( message.data );      
    }
  }
}






















/*
 
#include <Wire.h>
#include <SPI.h>
#include <MySensor.h>
#include <DHT.h>
#include <Bounce2.h>
#include <LiquidCrystal_I2C.h>


LiquidCrystal_I2C lcd(0x27,16,2);

#define NODE_ID 19
#define SKETCH_NAME "MySensor_HVAC"
#define SKETCH_VERSION "1.1"
#define NODE_REPEAT false

#define CHILD_ID_HUM 5
#define CHILD_ID_TEMP 7
#define CHILD_ID_HVAC 7

#define HUMIDITY_SENSOR_DIGITAL_PIN 3

MySensor gw;

DHT dht;
float lastTemp, lastHum, setpoint;
float dhttemperature =23.2, dhthumidity=40.7;
boolean metric = true;
unsigned long dht_lastMillis = 0; //Start time for waittoupdate function
unsigned long SLEEP_TIME = 30 * 1000; // Sleep time between DHT reads (in milliseconds)

MyMessage msgHum(CHILD_ID_HUM, V_HUM); //Vera device for Humidity
MyMessage msgTemp(CHILD_ID_HVAC, V_TEMP); //Vera device for Temperature
MyMessage msgFlowmode(CHILD_ID_HVAC, V_HVAC_FLOW_MODE);
//MyMessage msgSetpointCool(CHILD_ID_HVAC, V_HVAC_SETPOINT_COOL);
MyMessage msgSetpointHeat(CHILD_ID_HVAC, V_HVAC_SETPOINT_HEAT);
MyMessage msgSetpointFlowState(CHILD_ID_HVAC, V_HVAC_FLOW_STATE);

String heaterMode = "Off";
float setPoint = 70;
float recievedvalue;

void setup()
{

  lcd.init(); 
  lcd.backlight();
  lcd.setCursor(0,0);
  lcd.print("Hello, World!");

  //gw.begin();
  gw.begin(incomingMessage, NODE_ID, NODE_REPEAT);
  Serial.println("GW begin ");
  // Send the Sketch Version Information to the Gateway
  gw.sendSketchInfo(SKETCH_NAME, SKETCH_VERSION);
  
  Serial.println("GW send info ");
  
  gw.present(CHILD_ID_HUM, S_HUM);
  //gw.present(CHILD_ID_TEMP, S_TEMP);
  gw.present(CHILD_ID_HVAC, S_HVAC);

  metric = gw.getConfig().isMetric;

  
  
}


void loop()
{
  gw.process();
  processdht();
 

}

void incomingMessage(const MyMessage &message)
{
  if (message.type == V_HVAC_SETPOINT_HEAT)
  {
    Serial.println("Setpoint Heat Message received");
    Serial.println(String(message.data));

    float recievedvalue = String(message.data).toFloat();

    if (recievedvalue > 0)
    {
      setpoint=recievedvalue;
      delay(50);
      gw.send(msgSetpointHeat.set(recievedvalue, 1));
      //Store state in eeprom
      delay(50);
      gw.saveState(CHILD_ID_HVAC, recievedvalue);
     
      Serial.println("done...");
    }
    

    
  }
  if (message.type == V_HVAC_FLOW_STATE)
  {
    Serial.println("Flow State Message received");
    Serial.println(String(message.data));

  loadsetpoint();
    
  }

 // if (message.type == V_HVAC_SETPOINT_COOL)
  //{
  //  Serial.println("Setpoint Cool Message received");
  //  Serial.println(String(message.data));
 // }

  if (message.type == V_HVAC_FLOW_MODE)
  {
    Serial.println("Flow Mode Message received");
    Serial.println(String(message.data));
    loadsetpoint();
  }
  
}

void loadsetpoint(){

  if (setpoint <= 0)
      setpoint = 23.5; // gw.loadState(CHILD_ID_HVAC);
    

  if (setpoint > 0)
    {
      delay(50);
      gw.send(msgSetpointHeat.set(setpoint, 1));
      delay(50);
      Serial.println("done...set");
    }
}

void processdht() {
  unsigned long dht_currentMillis = millis();

  if ((dht_currentMillis - dht_lastMillis) > SLEEP_TIME ) { //sleep a bit

    dht_lastMillis = dht_currentMillis;
    
    gw.send(msgTemp.set(dhttemperature, 1));
    Serial.print("T: ");
    Serial.println(dhttemperature);

    gw.send(msgHum.set(dhthumidity, 1));
    Serial.print("H: ");
    Serial.println(dhthumidity);

    loadsetpoint();
   
  }
}




#define CHILD_ID_HEATER 4

//#define ONE_WIRE_BUS 3 // Pin where dallase sensor is connected 
//#define MAX_ATTACHED_DS18B20 2

#define RELAY_PIN  4  // Arduino Digital I/O pin number for relay 
#define BUTTON_PIN  5  // Arduino Digital I/O pin number for button 
#define UP_BUTTON_PIN  6  // Arduino Digital I/O pin number for button
#define DOWN_BUTTON_PIN  7  // Arduino Digital I/O pin number for button
#define RELAY_ON 1
#define RELAY_OFF 0
unsigned long SLEEP_TIME = 60000; // Sleep time between reads (in milliseconds)

OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);

Bounce debouncer1 = Bounce();
Bounce debouncer2 = Bounce();
Bounce debouncer3 = Bounce();
int oldValue = 0;
int oldupValue = 0;
int olddownValue = 0;
int BacklightStatus = 0;
bool state;
String SWstate = "Off";
MySensor gw;


int numSensors = 0;

float lastTemperature[MAX_ATTACHED_DS18B20];
float temperature = 0;

boolean metric = true;

unsigned long CHECK_TIME = millis();
unsigned long DISP_CHECK_TIME = millis();
unsigned long DISP_ON_TIME = 10000;

String heaterMode = "Off";
float setPoint = 70;
float recievedvalue;

MyMessage msgTemp(0, V_TEMP);
MyMessage msgHeater(CHILD_ID_HEATER, V_HVAC_FLOW_STATE);
MyMessage msgSetpoint(CHILD_ID_HEATER, V_HVAC_SETPOINT_HEAT);


void setup()
{
  Serial.println("setup ");
  // Startup OneWire
  sensors.begin();
  Serial.println("Sensor begin ");
  gw.begin(incomingMessage, NODE_ID, NODE_REPEAT);
  Serial.println("GW begin ");
  // Send the Sketch Version Information to the Gateway
  gw.sendSketchInfo(SKETCH_NAME, SKETCH_VERSION);
  
  Serial.println("GW send info ");
  
  // Setup the button
  pinMode(BUTTON_PIN, INPUT);
  pinMode(UP_BUTTON_PIN, INPUT);
  pinMode(DOWN_BUTTON_PIN, INPUT);
  // Activate internal pull-up
  digitalWrite(BUTTON_PIN, HIGH);
  digitalWrite(UP_BUTTON_PIN, HIGH);
  digitalWrite(DOWN_BUTTON_PIN, HIGH);

  // After setting up the button, setup debouncers
  debouncer1.attach(BUTTON_PIN);
  debouncer1.interval(5);
  debouncer2.attach(UP_BUTTON_PIN);
  debouncer2.interval(5);
  debouncer3.attach(DOWN_BUTTON_PIN);
  debouncer3.interval(5);

  numSensors = 1; //sensors.getDeviceCount();

  // Register all sensors to gw (they will be created as child devices)

  // Present 1-wire sensors to controller
  for (int i = 0; i < numSensors && i < MAX_ATTACHED_DS18B20; i++)
  {
    gw.present(i, S_TEMP);
  }

  //gw.present(CHILD_ID_HEATER, S_HEATER);
  gw.present(CHILD_ID_HEATER, S_HVAC);
  metric = gw.getConfig().isMetric;

  lcd.begin(16, 2); // initialize the lcd

  lcd.backlight();

  // Make sure relays are off when starting up
  digitalWrite(RELAY_PIN, RELAY_OFF);
  delay(500);
  // Then set relay pins in output mode
  pinMode(RELAY_PIN, OUTPUT);

  // Set setPoint to last known state (using eeprom storage)
  setPoint = gw.loadState(CHILD_ID_HEATER);
  Serial.print("Last known SetPoint: ");
  Serial.println(setPoint);


  lcd.setCursor(0, 0);
  lcd.print("Hej!");// You can put a custome initialization message to display

  delay(2000);
  lcd.clear();

  delay(10);

  if (setPoint > 120)
  {
    setPoint = 71;
    gw.send(msgSetpoint.set(setPoint, 1));
    gw.saveState(CHILD_ID_HEATER, setPoint);
  }

  updateSetTemp();
  lcd.noBacklight();
}

void loop()
{
  gw.process();
  debouncer1.update();
  debouncer2.update();
  debouncer3.update();
  // Get the update value
  int value = debouncer1.read();
  int upValue = debouncer2.read();
  int downValue = debouncer3.read();

  if (value != oldValue && value == 0)
  {
    bool executed = 0;

    if (SWstate == "Off" && executed == 0)
    {
      Serial.println("Heating ON by button");
      digitalWrite(RELAY_PIN, RELAY_ON);
      gw.wait(500);
      SWstate = "HeatOn";
      Serial.println("Sending value to GW: ");
      Serial.println(SWstate);
      gw.send(msgHeater.set(SWstate.c_str()), true);
      BacklightStatus = 1;
      lcd.backlight();
      executed = 1;

    }

    if (SWstate == "HeatOn" && executed == 0)
    {
      Serial.println("Heating OFF by button");
      digitalWrite(RELAY_PIN, RELAY_OFF);
      gw.wait(500);
      SWstate = "Off";
      Serial.print("Sending value to GW: ");
      Serial.println(SWstate);
      gw.send(msgHeater.set(SWstate.c_str()), true);
      BacklightStatus = 1;
      lcd.backlight();
      executed = 1;
    }

  }
  oldValue = value;



  if ((upValue != oldupValue && upValue == 0) || (downValue != olddownValue && downValue == 0))
  {
    if (upValue == 0)
    {
      setPoint = setPoint + 0.5;
      updateSetTemp();
      gw.send(msgSetpoint.set(setPoint, 1));
      BacklightStatus = 1;
      lcd.backlight();
    }

    if (downValue == 0)
    {
      setPoint = setPoint - 0.5;
      updateSetTemp();
      gw.send(msgSetpoint.set(setPoint, 1));
      BacklightStatus = 1;
      lcd.backlight();
    }

  }
  oldupValue = upValue;
  olddownValue = downValue;

  // Check Temp sensor every set ammount of time
  unsigned long NOW_TIME = millis();
  if(NOW_TIME - CHECK_TIME >= SLEEP_TIME)
  {
    getTemps();
    updateTemp();
    CHECK_TIME = NOW_TIME;
  }
  if (BacklightStatus == 1)
  {
    unsigned long DISP_NOW_TIME = millis();

    if((DISP_NOW_TIME - DISP_CHECK_TIME >= DISP_ON_TIME) && (BacklightStatus == 1))
    {

      DISP_CHECK_TIME = DISP_NOW_TIME;
      BacklightStatus == 0;
      lcd.noBacklight();
    }

  }
  
}

void getTemps()
{

  // Fetch temperatures from Dallas sensors
  sensors.requestTemperatures();

  // Read temperatures and send them to controller
  for (int i = 0; i < numSensors && i < MAX_ATTACHED_DS18B20; i++)
  {

    // Fetch and round temperature to one decimal
    temperature = static_cast<float>(static_cast<int>((gw.getConfig().isMetric ? sensors.getTempCByIndex(i) : sensors.getTempFByIndex(i)) * 10.)) / 10.;

    // Only send data if temperature has changed and no error
    if (lastTemperature[i] != temperature && temperature != -127.00)
    {

      // Send in the new temperature
      gw.send(msgTemp.setSensor(CHILD_ID_HEATER).set(temperature, 1));
      gw.send(msgTemp.setSensor(i).set(temperature, 1));
      Serial.print("T: ");
      Serial.println(temperature);
      lastTemperature[i] = temperature;
    }
  }

  if (SWstate == "Off" && (temperature - 1 > setPoint))
  {
    Serial.println("Heating ON by temperature");
    digitalWrite(RELAY_PIN, RELAY_ON);
    gw.wait(500);
    SWstate = "HeatOn";
    Serial.println("Sending new switch state to GW: ");
    Serial.println(SWstate);
    gw.send(msgHeater.set(SWstate.c_str()), true);
    BacklightStatus = 1;
    lcd.backlight();
  }

  if (SWstate == "HeatOn" && (temperature + 1 < setPoint))
  {
    Serial.println("Heating OFF by temperature");
    digitalWrite(RELAY_PIN, RELAY_OFF);
    gw.wait(500);
    SWstate = "Off";
    Serial.println("Sending new switch state to GW: ");
    Serial.println(SWstate);
    gw.send(msgHeater.set(SWstate.c_str()), true);
    BacklightStatus = 1;
    lcd.backlight();
  }

}

void incomingMessage(const MyMessage &message)
{
  if (message.type == V_HVAC_SETPOINT_HEAT)
  {
    
    recievedvalue = String(message.data).toFloat();

    
    if (recievedvalue > 0)
    {
      Serial.println(String(message.data));
      setPoint = recievedvalue;
      Serial.println("New SetPoint Temp recieved: " + String(setPoint));
      gw.send(msgSetpoint.set(setPoint, 1));
      //Store state in eeprom
      gw.saveState(CHILD_ID_HEATER, setPoint);
      updateSetTemp();
      BacklightStatus = 1;
      lcd.backlight();
    }
  }
  if (message.type == V_HVAC_FLOW_STATE)
  {
    Serial.println("Flow State Message received");
    Serial.println(String(message.data));


    if(String(message.data) == "HeatOn")
    {
      digitalWrite(RELAY_PIN, RELAY_ON);
      gw.wait(500);
      SWstate = "HeatOn";
      BacklightStatus = 1;
      lcd.backlight();
    }

    if(String(message.data) == "Off")
    {
      digitalWrite(RELAY_PIN, RELAY_OFF);
      gw.wait(500);
      SWstate = "Off";
      BacklightStatus = 1;
      lcd.backlight();
    }
  }
}

void updateTemp()
{
  lcd.setCursor(0, 0);
  lcd.print("T: ");
  lcd.print(temperature);
  lcd.write(223); // Degree-sign
  lcd.print("C      ");
}

void updateSetTemp()
{
  lcd.setCursor(0, 1);
  lcd.print("SetT: ");
  lcd.print(setPoint);
  lcd.write(223); // Degree-sign
  lcd.print("C   ");
}

*/
